@@ -1,12 +1,12 @@
 # DIF MANTEL-HAENZEL
 
-difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE, exact=FALSE,
+difMH<-function (Data, group, focal.name, anchor=NULL, MHstat = "MHChisq", correct = TRUE, exact=FALSE,
     alpha = 0.05, purify = FALSE, nrIter = 10, save.output = FALSE, 
     output = c("out", "default")) 
 {
     internalMH <- function() {
         if (length(group) == 1) {
-            if (is.numeric(group) == TRUE) {
+            if (is.numeric(group)) {
                 gr <- Data[, group]
                 DATA <- Data[, (1:ncol(Data)) != group]
                 colnames(DATA) <- colnames(Data)[(1:ncol(Data)) != 
@@ -28,16 +28,35 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE, ex
         Q <- switch(MHstat, MHChisq = qchisq(1 - alpha, 1), logOR = qnorm(1 - 
             alpha/2))
         if (is.null(Q)) stop("'MHstat' argument not valid", call. = FALSE)
-
+if (!is.null(anchor)){
+dif.anchor<-anchor
+if (is.numeric(anchor)) ANCHOR<-anchor
+else{
+ANCHOR<-NULL
+for (i in 1:length(anchor)) ANCHOR[i]<-(1:ncol(DATA))[colnames(DATA)==anchor[i]]
+}
+}
+else {
+ANCHOR<-1:ncol(DATA)
+dif.anchor<-NULL
+}
 if (exact){
-        if (!purify) {
-            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
+        if (!purify | !is.null(anchor)) {
+            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact,anchor=ANCHOR)
                 STATS <- PROV$resMH
             if (min(PROV$Pval) >=alpha) DIFitems <- "No DIF item detected"
             else DIFitems <- (1:ncol(DATA))[PROV$Pval < alpha]
             RES <- list(MH = STATS, Pval=PROV$Pval, alpha = alpha, DIFitems = DIFitems, 
                 correct = correct, exact=exact, purification = purify, names = colnames(DATA), 
-                save.output = save.output, output = output)
+                anchor.names=dif.anchor,save.output = save.output, output = output)
+if (!is.null(anchor)) {
+RES$MH[ANCHOR]<-NA
+RES$Pval[ANCHOR]<-NA
+for (i in 1:length(RES$DIFitems)){
+if (sum(RES$DIFitems[i]==ANCHOR)==1) RES$DIFitems[i]<-NA
+}
+RES$DIFitems<-RES$DIFitems[!is.na(RES$DIFitems)]
+}
         }
         else {
             nrPur <- 0
@@ -103,13 +122,12 @@ if (exact){
             RES <- list(MH = stats1, Pval=prov1$Pval, alpha = alpha, DIFitems = DIFitems, 
                 correct = correct, exact=exact, purification = purify, nrPur = nrPur, 
                 difPur = difPur, convergence = noLoop, names = colnames(DATA), 
-                save.output = save.output, output = output)
+                anchor.names=NULL, save.output = save.output, output = output)
         }
 }
 else{
-
-        if (!purify) {
-            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
+        if (!purify | !is.null(anchor)) {
+            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact,anchor=ANCHOR)
             if (MHstat == "MHChisq") 
                 STATS <- PROV$resMH
             else STATS <- log(PROV$resAlpha)/sqrt(PROV$varLambda)
@@ -120,7 +138,16 @@ else{
                 varLambda = PROV$varLambda, MHstat = MHstat, 
                 alpha = alpha, thr = Q, DIFitems = DIFitems, 
                 correct = correct, exact=exact, purification = purify, names = colnames(DATA), 
-                save.output = save.output, output = output)
+                anchor.names=dif.anchor, save.output = save.output, output = output)
+if (!is.null(anchor)) {
+RES$MH[ANCHOR]<-NA
+RES$alphaMH[ANCHOR]<-NA
+RES$varLambda[ANCHOR]<-NA
+for (i in 1:length(RES$DIFitems)){
+if (sum(RES$DIFitems[i]==ANCHOR)==1) RES$DIFitems[i]<-NA
+}
+RES$DIFitems<-RES$DIFitems[!is.na(RES$DIFitems)]
+}
         }
         else {
             nrPur <- 0
@@ -194,7 +221,7 @@ else{
                 alpha = alpha, thr = qchisq(1 - alpha, 1), DIFitems = DIFitems, 
                 correct = correct, exact=exact,purification = purify, nrPur = nrPur, 
                 difPur = difPur, convergence = noLoop, names = colnames(DATA), 
-                save.output = save.output, output = output)
+                anchor.names=NULL, save.output = save.output, output = output)
         }
 }
         class(RES) <- "MH"
@@ -220,15 +247,15 @@ if (x$exact) stop("Error: plot is not available with exact Mantel-Haenszel test"
 internalMH<-function(){
     res <- x
     if (res$MHstat == "MHChisq") 
-        yl <- c(0, max(c(res$MH, res$thr) + 1))
-    else yl <- c(min(c(res$MH, -res$thr) - 0.5), max(c(res$MH, 
-        res$thr) + 0.5))
+        yl <- c(0, max(c(res$MH, res$thr) + 1,na.rm=TRUE))
+    else yl <- c(min(c(res$MH, -res$thr) - 0.5,na.rm=TRUE), max(c(res$MH, 
+        res$thr) + 0.5,na.rm=TRUE))
     ytitle = switch(res$MHstat, MHChisq = "MH Chi-square statistic", 
         logOR = "log OR statistic")
-    if (number == FALSE) {
+    if (!number) {
         plot(res$MH, xlab = "Item", ylab = ytitle, ylim = yl, 
             pch = pch, main = "Mantel-Haenszel")
-        if (is.character(res$DIFitems) == FALSE) 
+        if (!is.character(res$DIFitems)) 
             points(res$DIFitems, res$MH[res$DIFitems], pch = pch, 
                 col = col)
     }
@@ -236,7 +263,7 @@ internalMH<-function(){
         plot(res$MH, xlab = "Item", ylab = ytitle, ylim = yl, 
             col = "white", main = "Mantel-Haenszel")
         text(1:length(res$MH), res$MH, 1:length(res$MH))
-        if (is.character(res$DIFitems) == FALSE) 
+        if (!is.character(res$DIFitems)) 
             text(res$DIFitems, res$MH[res$DIFitems], res$DIFitems, 
                 col = col)
     }
@@ -245,11 +272,11 @@ internalMH<-function(){
         abline(h = -res$thr)
 }
 internalMH()
-if (save.plot==TRUE){
+if (save.plot){
 plotype<-NULL
 if (save.options[3]=="pdf") plotype<-1
 if (save.options[3]=="jpeg") plotype<-2
-if (is.null(plotype)==TRUE) cat("Invalid plot type (should be either 'pdf' or 'jpeg').","\n","The plot was not captured!","\n")
+if (is.null(plotype)) cat("Invalid plot type (should be either 'pdf' or 'jpeg').","\n","The plot was not captured!","\n")
 else {
 if (save.options[2]=="default") wd<-paste(getwd(),"/",sep="")
 else wd<-save.options[2]
@@ -282,17 +309,17 @@ print.MH<-function (x, ...)
     cat("\n")
     cat("Detection of Differential Item Functioning using Mantel-Haenszel method", 
         "\n")
-    if (res$correct == TRUE) 
+    if (res$correct & !res$exact) 
         corr <- "with "
     else corr <- "without "
-    if (res$purification == TRUE) 
+    if (res$purification & is.null(res$anchor.names)) 
         pur <- "with "
     else pur <- "without "
     cat(corr, "continuity correction and ", pur, "item purification", 
         "\n", "\n", sep = "")
     if (res$exact) cat("Results based on exact inference","\n","\n")
     else cat("Results based on asymptotic inference","\n","\n")
-    if (res$purification) {
+    if (res$purification & is.null(res$anchor.names)) {
         if (res$nrPur <= 1) 
             word <- " iteration"
         else word <- " iterations"
@@ -312,6 +339,21 @@ print.MH<-function (x, ...)
         else cat("Convergence reached after ", res$nrPur, word, 
             "\n", "\n", sep = "")
     }
+if (is.null(res$anchor.names)) {
+itk<-1:length(res$MH)
+cat("No set of anchor items was provided", "\n", "\n")
+}
+else {
+itk<-(1:length(res$MH))[!is.na(res$MH)]
+cat("Anchor items (provided by the user):", "\n")
+if (is.numeric(res$anchor.names)) mm<-res$names[res$anchor.names]
+else mm<-res$anchor.names
+mm <- cbind(mm)
+rownames(mm) <- rep("", nrow(mm))
+colnames(mm) <- ""
+print(mm, quote = FALSE)
+cat("\n", "\n")
+}
 if (res$exact) met<-"Exact statistic:"
 else    met <- switch(res$MHstat, MHChisq = "Mantel-Haenszel Chi-square statistic:", 
         logOR = "Log odds-ratio statistic:")
@@ -323,14 +365,14 @@ else{
 }
     symb <- symnum(pval, c(0, 0.001, 0.01, 0.05, 0.1, 1), symbols = c("***", 
         "**", "*", ".", ""))
-    if (!res$exact) m1 <- cbind(round(res$MH, 4), pval)
-else m1 <- cbind(round(res$MH), pval)
-    m1 <- noquote(cbind(format(m1, justify = "right"), symb))
-    if (!is.null(res$names)) rownames(m1) <- res$names
+    if (!res$exact) m1 <- cbind(round(res$MH[itk], 4), pval[itk])
+else m1 <- cbind(round(res$MH[itk]), pval[itk])
+    m1 <- noquote(cbind(format(m1, justify = "right"), symb[itk]))
+    if (!is.null(res$names)) rownames(m1) <- res$names[itk]
     else {
         rn <- NULL
         for (i in 1:nrow(m1)) rn[i] <- paste("Item", i, sep = "")
-        rownames(m1) <- rn
+        rownames(m1) <- rn[itk]
     }
     colnames(m1) <- c("Stat.", "P-value", "")
     print(m1)
@@ -343,7 +385,13 @@ else cat("\n", "Detection threshold: ", round(res$thr, 4), " (significance level
     if (is.character(res$DIFitems)) cat("Items detected as DIF items:", res$DIFitems, "\n", "\n")
     else {
         cat("Items detected as DIF items:", "\n")
-        m2 <- cbind(rownames(m1)[res$DIFitems])
+   if (!is.null(res$names)) m2 <- res$names
+    else {
+        rn <- NULL
+        for (i in 1:length(res$MH)) rn[i] <- paste("Item", i, sep = "")
+        m2 <- rn
+    }
+        m2 <- cbind(m2[res$DIFitems])
         rownames(m2) <- rep("", nrow(m2))
         colnames(m2) <- ""
         print(m2, quote = FALSE)
@@ -358,15 +406,14 @@ if (!res$exact){
     r2 <- round(-2.35 * log(res$alphaMH), 4)
     symb1 <- symnum(abs(r2), c(0, 1, 1.5, Inf), symbols = c("A", 
         "B", "C"))
-    matR2 <- cbind(round(res$alphaMH, 4), r2)
+    matR2 <- cbind(round(res$alphaMH[itk], 4), r2[itk])
     matR2 <- noquote(cbind(format(matR2, justify = "right"), 
-        symb1))
-    if (is.null(res$names) == FALSE) 
-        rownames(matR2) <- res$names
+        symb1[itk]))
+    if (!is.null(res$names)) rownames(matR2) <- res$names[itk]
     else {
         rn <- NULL
         for (i in 1:nrow(matR2)) rn[i] <- paste("Item", i, sep = "")
-        rownames(matR2) <- rn
+        rownames(matR2) <- rn[itk]
     }
     colnames(matR2) <- c("alphaMH", "deltaMH", "")
     print(matR2)
@@ -384,7 +431,3 @@ cat("Output was captured and saved into file","\n"," '",fileName,"'","\n","\n",s
 }
 
 
-
-
-
-
